@@ -189,10 +189,11 @@ public:
             wxASSERT_MSG( m_rectAll.width <= w,
                             wxT("width can only be increased") );
 
+            int delta = (w - m_rectAll.width) / 2;
             m_rectAll.width = w;
-            m_rectLabel.x += (w - m_rectLabel.width) / 2;
-            m_rectIcon.x += (w - m_rectIcon.width) / 2;
-            m_rectHighlight.x += (w - m_rectHighlight.width) / 2;
+            m_rectLabel.x += delta;
+            m_rectIcon.x += delta;
+            m_rectHighlight.x += delta;
         }
     }
     *m_gi;
