@@ -1005,9 +1005,34 @@ wxCopyFile (const wxString& file1, const wxString& file2, bool overwrite)
 bool
 wxRenameFile(const wxString& file1, const wxString& file2, bool overwrite)
 {
+#ifdef __WINDOWS__
+    // When overwriting, prefer using ReplaceFile() which allows to preserve
+    // the destination file attributes while replacing its contents.
+    if ( overwrite && wxFileExists(file2) )
+    {
+        if ( ::ReplaceFile
+               (
+                    file2.t_str(),  // File to replace.
+                    file1.t_str(),  // File to replace it with.
+                    NULL,           // No backup file.
+                    REPLACEFILE_IGNORE_MERGE_ERRORS,
+                                    // Don't return error just because ACLs
+                                    // couldn't be preserved.
+                    wxRESERVED_PARAM,
+                    wxRESERVED_PARAM
+               ) )
+        {
+            return true;
+        }
+
+        // Continue with the normal rename logic if ReplaceFile() failed, it
+        // will probably fail as well but it shouldn't hurt to try.
+    }
+#endif // __WINDOWS__
+
     if ( !overwrite && wxFileExists(file2) )
     {
-        wxLogSysError
+        wxLogError
         (
             _("Failed to rename the file '%s' to '%s' because the destination file already exists."),
             file1.c_str(), file2.c_str()
