@@ -261,7 +261,34 @@ void wxFileDialog::ShowWindowModal()
 
     wxCHECK_RET(parentWindow, "Window modal display requires parent.");
 
-    NSArray* types = GetTypesFromFilter( m_wildCard, m_filterNames, m_filterExtensions, m_currentExtensions ) ;
+    NSArray* allTypes = GetTypesFromFilter( m_wildCard, m_filterNames, m_filterExtensions, m_currentExtensions ) ;
+
+    m_useFileTypeFilter = m_filterExtensions.GetCount() > 1;
+
+    // native behaviour on macos is to enable just every file type
+    // in a file open dialog that could be handled, without a file
+    // filter choice control
+    // wxOSX_FILEDIALOG_ALWAYS_SHOW_TYPES allows to override
+    // that and get the same behaviour as other platforms have ...
+
+    if( HasFlag(wxFD_OPEN) )
+    {
+      if ( !(wxSystemOptions::HasOption( wxOSX_FILEDIALOG_ALWAYS_SHOW_TYPES ) && (wxSystemOptions::GetOptionInt( wxOSX_FILEDIALOG_ALWAYS_SHOW_TYPES ) == 1)) )
+        m_useFileTypeFilter = false;
+    }
+
+    m_firstFileTypeFilter = -1;
+
+    if ( m_useFileTypeFilter
+        && m_filterIndex >= 0 && (size_t)m_filterIndex < m_filterExtensions.GetCount() )
+    {
+      m_firstFileTypeFilter = m_filterIndex;
+    }
+    else if ( m_useFileTypeFilter )
+    {
+      m_firstFileTypeFilter = GetMatchingFilterExtension(m_fileName);
+    }
+
     if ( HasFlag(wxFD_SAVE) )
     {
         NSSavePanel* sPanel = [NSSavePanel savePanel];
@@ -275,10 +302,33 @@ void wxFileDialog::ShowWindowModal()
         // be able to pass this in
         [sPanel setTreatsFilePackagesAsDirectories:NO];
         [sPanel setCanSelectHiddenExtension:YES];
-        [sPanel setAllowedFileTypes:types];
+        [sPanel setAllowedFileTypes:allTypes];
         [sPanel setAllowsOtherFileTypes:NO];
         [sPanel setShowsHiddenFiles: HasFlag(wxFD_SHOW_HIDDEN) ? YES : NO];
 
+        /*
+         Let the file dialog know what file type should be used initially.
+         If this is not done then when setting the filter index
+         programmatically to 1 the file will still have the extension
+         of the first file type instead of the second one. E.g. when file
+         types are foo and bar, a filename "myletter" with SetDialogIndex(1)
+         would result in saving as myletter.foo, while we want myletter.bar.
+         */
+        if(m_firstFileTypeFilter > 0)
+        {
+            DoOnFilterSelected(m_firstFileTypeFilter);
+        }
+        else
+        {
+            NSArray* types = GetTypesFromExtension(m_filterExtensions[m_firstFileTypeFilter], m_currentExtensions);
+            if ( m_delegate )
+                [(wxOpenSavePanelDelegate*) m_delegate setAllowedExtensions: m_currentExtensions];
+            else
+            {
+                [sPanel setAllowedFileTypes: types];
+            }
+        }
+
         NSWindow* nativeParent = parentWindow->GetWXWindow();
         if ( !m_dir.IsEmpty() )
             [sPanel setDirectoryURL:[NSURL fileURLWithPath:dir.AsNSString()
@@ -304,8 +354,24 @@ void wxFileDialog::ShowWindowModal()
         [oPanel setCanChooseFiles:YES];
         [oPanel setMessage:cf.AsNSString()];
         [oPanel setAllowsMultipleSelection: (HasFlag(wxFD_MULTIPLE) ? YES : NO )];
+        [oPanel setAllowedFileTypes:allTypes];
+        [oPanel setAllowsOtherFileTypes:NO];
         [oPanel setShowsHiddenFiles: HasFlag(wxFD_SHOW_HIDDEN) ? YES : NO];
 
+        // Note that the test here is intentionally different from the one
+        // above, in the wxFD_SAVE case: we need to call DoOnFilterSelected()
+        // even for m_firstFileTypeFilter == 0, i.e. when using the default
+        // filter.
+        if ( m_firstFileTypeFilter >= 0 )
+        {
+          DoOnFilterSelected(m_firstFileTypeFilter);
+        }
+        else
+        {
+          if ( m_delegate )
+            [(wxOpenSavePanelDelegate*) m_delegate setAllowedExtensions: m_currentExtensions];
+        }
+
         NSWindow* nativeParent = parentWindow->GetWXWindow();
         if ( !m_dir.IsEmpty() )
             [oPanel setDirectoryURL:[NSURL fileURLWithPath:dir.AsNSString()
@@ -398,7 +464,9 @@ void wxFileDialog::DoOnFilterSelected(int index)
         [panel validateVisibleColumns];
     }
     else
+    {
         [panel setAllowedFileTypes:types];
+    }
 
     m_currentlySelectedFilterIndex = index;
 
@@ -447,8 +515,12 @@ void wxFileDialog::SetupExtraControls(WXWindow nativeWindow)
         [accView removeFromSuperview];
         [panel setAccessoryView:accView];
 
+        // We need to explicitly show the accessory view for "Open" file
+        // dialogs, but doing it for "Save" dialogs is not only unnecessary but
+        // results in an immediate application abort (see #25717).
         wxCLANG_WARNING_SUPPRESS(undeclared-selector)
-        if ([panel respondsToSelector:@selector(setAccessoryViewDisclosed)])
+        if (!HasFlag(wxFD_SAVE) &&
+                [panel respondsToSelector:@selector(setAccessoryViewDisclosed:)])
         {
             [(id)panel setAccessoryViewDisclosed:YES];
         }
@@ -509,7 +581,7 @@ int wxFileDialog::ShowModal()
 
     int returnCode = -1;
 
-    NSArray* types = GetTypesFromFilter( m_wildCard, m_filterNames, m_filterExtensions, m_currentExtensions ) ;
+    NSArray* allTypes = GetTypesFromFilter( m_wildCard, m_filterNames, m_filterExtensions, m_currentExtensions ) ;
 
     m_useFileTypeFilter = m_filterExtensions.GetCount() > 1;
 
@@ -535,7 +607,6 @@ int wxFileDialog::ShowModal()
     else if ( m_useFileTypeFilter )
     {
         m_firstFileTypeFilter = GetMatchingFilterExtension(m_fileName);
-        types = GetTypesFromExtension(m_filterExtensions[m_firstFileTypeFilter], m_currentExtensions);
     }
 
     OSXBeginModalDialog();
@@ -553,7 +624,7 @@ int wxFileDialog::ShowModal()
         // be able to pass this in
         [sPanel setTreatsFilePackagesAsDirectories:NO];
         [sPanel setCanSelectHiddenExtension:YES];
-        [sPanel setAllowedFileTypes:types];
+        [sPanel setAllowedFileTypes:allTypes];
         [sPanel setAllowsOtherFileTypes:NO];
         [sPanel setShowsHiddenFiles: HasFlag(wxFD_SHOW_HIDDEN) ? YES : NO];
 
@@ -573,6 +644,14 @@ int wxFileDialog::ShowModal()
         {
             DoOnFilterSelected(m_firstFileTypeFilter);
         }
+        else
+        {
+            NSArray* types = GetTypesFromExtension(m_filterExtensions[m_firstFileTypeFilter], m_currentExtensions);
+            if ( m_delegate )
+                [(wxOpenSavePanelDelegate*) m_delegate setAllowedExtensions: m_currentExtensions];
+            else
+                [sPanel setAllowedFileTypes: types];
+        }
 
         if ( !m_dir.IsEmpty() )
             [sPanel setDirectoryURL:[NSURL fileURLWithPath:dir.AsNSString()
@@ -600,6 +679,8 @@ int wxFileDialog::ShowModal()
         [oPanel setCanChooseFiles:YES];
         [oPanel setMessage:cf.AsNSString()];
         [oPanel setAllowsMultipleSelection: (HasFlag(wxFD_MULTIPLE) ? YES : NO )];
+        [oPanel setAllowedFileTypes:allTypes];
+        [oPanel setAllowsOtherFileTypes:NO];
         [oPanel setShowsHiddenFiles: HasFlag(wxFD_SHOW_HIDDEN) ? YES : NO];
 
         // Note that the test here is intentionally different from the one
@@ -614,9 +695,9 @@ int wxFileDialog::ShowModal()
         {
             if ( m_delegate )
                 [(wxOpenSavePanelDelegate*) m_delegate setAllowedExtensions: m_currentExtensions];
-            else
-                [oPanel setAllowedFileTypes: types];
         }
+
+
         if ( !m_dir.IsEmpty() )
             [oPanel setDirectoryURL:[NSURL fileURLWithPath:dir.AsNSString() 
                                                isDirectory:YES]];
