@@ -156,14 +156,6 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *item, size_t pos)
     // if we're already attached to the menubar, we must update it
     if ( IsAttached() && GetMenuBar()->IsAttached() )
     {
-        if ( item->IsSubMenu() )
-        {
-            item->GetSubMenu()->SetupBitmaps();
-        }
-        if ( !item->IsSeparator() )
-        {
-            item->UpdateItemBitmap();
-        }
         GetMenuBar()->Refresh();
     }
 #endif // wxUSE_MENUBAR
@@ -359,25 +351,30 @@ bool wxMenu::HandleCommandUpdateStatus( wxMenuItem* item )
 
 bool wxMenu::HandleCommandProcess( wxMenuItem* item )
 {
-    int menuid = item ? item->GetId() : 0;
+    wxCHECK_MSG( item, false, "must have a valid item" );
+
+    int menuid = item->GetId();
     bool processed = false;
     if (item->IsCheckable())
         item->Check( !item->IsChecked() ) ;
 
-    if ( SendEvent( menuid , item->IsCheckable() ? item->IsChecked() : -1 ) )
-        processed = true ;
-
-    if(!processed && item)
-    {
-        processed = item->GetPeer()->DoDefault();  
-    }
-    
+    // A bit counterintuitively, we call OSXAfterMenuEvent() _before_ calling
+    // the user defined handler. This is done to account for the case when this
+    // handler deletes the window, as it can possibly do.
     if (wxWindow* const w = GetInvokingWindow())
     {
         // Let the invoking window update itself if necessary.
         w->OSXAfterMenuEvent();
     }
 
+    if ( SendEvent( menuid , item->IsCheckable() ? item->IsChecked() : -1 ) )
+        processed = true ;
+
+    if(!processed)
+    {
+        processed = item->GetPeer()->DoDefault();  
+    }
+
     return processed;
 }
 
@@ -413,38 +410,17 @@ void wxMenu::HandleMenuClosed()
 void wxMenu::Attach(wxMenuBarBase *menubar)
 {
     wxMenuBase::Attach(menubar);
-
-    if (menubar->IsAttached())
-    {
-        SetupBitmaps();
-    }
 }
 #endif
 
 void wxMenu::SetInvokingWindow(wxWindow* win)
 {
     wxMenuBase::SetInvokingWindow(win);
-
-    if ( win )
-        SetupBitmaps();
 }
 
 void wxMenu::SetupBitmaps()
 {
-    for ( wxMenuItemList::compatibility_iterator node = m_items.GetFirst();
-          node;
-          node = node->GetNext() )
-    {
-        wxMenuItem *item = node->GetData();
-        if ( item->IsSubMenu() )
-        {
-            item->GetSubMenu()->SetupBitmaps();
-        }
-        if ( !item->IsSeparator() )
-        {
-            item->UpdateItemBitmap();
-        }
-    }
+    // unused, kept for ABI compatibility
 }
 
 #if wxUSE_MENUBAR
@@ -499,8 +475,8 @@ static wxMenu *CreateAppleMenu()
     if ( wxApp::s_macPreferencesMenuItemId != wxID_NONE )
     {
         appleMenu->Append( wxApp::s_macPreferencesMenuItemId,
-                           wxGETTEXT_IN_CONTEXT("macOS menu item", "Preferences...")
-                           + "\tCtrl+," );
+                           wxString::Format("%s\tCtrl+,",
+                                            wxGETTEXT_IN_CONTEXT("macOS menu item", "Preferences...")) );
         appleMenu->AppendSeparator();
     }
 
@@ -517,7 +493,8 @@ static wxMenu *CreateAppleMenu()
         hideLabel = wxGETTEXT_IN_CONTEXT("macOS menu item", "Hide Application");
     appleMenu->Append( wxID_OSX_HIDE, hideLabel + "\tCtrl+H" );
     appleMenu->Append( wxID_OSX_HIDEOTHERS,
-                       wxGETTEXT_IN_CONTEXT("macOS menu item", "Hide Others")+"\tAlt+Ctrl+H" );
+                       wxString::Format("%s\tAlt+Ctrl+H",
+                                        wxGETTEXT_IN_CONTEXT("macOS menu item", "Hide Others")) );
     appleMenu->Append( wxID_OSX_SHOWALL,
                        wxGETTEXT_IN_CONTEXT("macOS menu item", "Show All") );
     appleMenu->AppendSeparator();
@@ -688,17 +665,12 @@ wxString wxMenuBar::GetMenuLabel(size_t pos) const
 
 void wxMenuBar::SetupBitmaps()
 {
-    for ( wxMenuList::const_iterator it = m_menus.begin(); it != m_menus.end(); ++it )
-    {
-        (*it)->SetupBitmaps();
-    }
+    // unused, kept for ABI compatibility
 }
 
 void wxMenuBar::Attach(wxFrame *frame)
 {
     wxMenuBarBase::Attach(frame);
-
-    SetupBitmaps();
 }
 
 // ---------------------------------------------------------------------------
